@@ -29,12 +29,15 @@ from mapproxy.compat.modules import urlparse
 
 if PY2:
     import urllib2
-    from urllib2 import URLError, HTTPError
+    from urllib2 import URLError, HTTPError, HTTPCookieProcessor
     import httplib
+    from cookielib import CookieJar
 else:
     from urllib import request as urllib2
     from urllib.error import URLError, HTTPError
+    from urllib.request import HTTPCookieProcessor
     from http import client as httplib
+    from http.cookiejar import CookieJar
 
 import socket
 import ssl
@@ -45,9 +48,10 @@ if hasattr(ssl, 'create_default_context'):
     supports_ssl_default_context = True
 
 class HTTPClientError(Exception):
-    def __init__(self, arg, response_code=None):
+    def __init__(self, arg, response_code=None, full_msg=None):
         Exception.__init__(self, arg)
         self.response_code = response_code
+        self.full_msg = full_msg
 
 
 def build_https_handler(ssl_ca_certs, insecure):
@@ -130,8 +134,8 @@ class _URLOpenerCache(object):
     def __init__(self):
         self._opener = {}
 
-    def __call__(self, ssl_ca_certs, url, username, password, insecure=False):
-        cache_key = (ssl_ca_certs, insecure)
+    def __call__(self, ssl_ca_certs, url, username, password, insecure=False, manage_cookies=False):
+        cache_key = (ssl_ca_certs, insecure, manage_cookies)
         if cache_key not in self._opener:
             handlers = []
             https_handler = build_https_handler(ssl_ca_certs, insecure)
@@ -142,6 +146,9 @@ class _URLOpenerCache(object):
             handlers.append(authhandler)
             authhandler = urllib2.HTTPDigestAuthHandler(passman)
             handlers.append(authhandler)
+            if manage_cookies:
+                cj = CookieJar()
+                handlers.append(HTTPCookieProcessor(cj))
 
             opener = urllib2.build_opener(*handlers)
 
@@ -160,7 +167,8 @@ create_url_opener = _URLOpenerCache()
 
 class HTTPClient(object):
     def __init__(self, url=None, username=None, password=None, insecure=False,
-                 ssl_ca_certs=None, timeout=None, headers=None):
+                 ssl_ca_certs=None, timeout=None, headers=None, hide_error_details=False,
+                 manage_cookies=False):
         self._timeout = timeout
         if url and url.startswith('https'):
             if insecure:
@@ -169,19 +177,22 @@ class HTTPClient(object):
                     raise HTTPClientError('No ca_certs file set (http.ssl_ca_certs). '
                         'Set file or disable verification with http.ssl_no_cert_checks option.')
 
-        self.opener = create_url_opener(ssl_ca_certs, url, username, password, insecure=insecure)
+        self.opener = create_url_opener(ssl_ca_certs, url, username, password, insecure=insecure, manage_cookies=manage_cookies)
         self.header_list = headers.items() if headers else []
+        self.hide_error_details = hide_error_details
 
-    def open(self, url, data=None):
+    def open(self, url, data=None, method=None):
         code = None
         result = None
         try:
             req = urllib2.Request(url, data=data)
         except ValueError as e:
-            reraise_exception(HTTPClientError('URL not correct "%s": %s'
-                                              % (url, e.args[0])), sys.exc_info())
+            err = self.handle_url_exception(url, 'URL not correct', e.args[0])
+            reraise_exception(err, sys.exc_info())
         for key, value in self.header_list:
             req.add_header(key, value)
+        if method:
+            req.method=method
         try:
             start_time = time.time()
             if self._timeout is not None:
@@ -190,25 +201,24 @@ class HTTPClient(object):
                 result = self.opener.open(req)
         except HTTPError as e:
             code = e.code
-            reraise_exception(HTTPClientError('HTTP Error "%s": %d'
-                % (url, e.code), response_code=code), sys.exc_info())
+            err = self.handle_url_exception(url, 'HTTP Error', str(code), response_code=code)
+            reraise_exception(err, sys.exc_info())
         except URLError as e:
             if isinstance(e.reason, ssl.SSLError):
-                e = HTTPClientError('Could not verify connection to URL "%s": %s'
-                                     % (url, e.reason.args[1]))
-                reraise_exception(e, sys.exc_info())
+                err = self.handle_url_exception(url, 'Could not verify connection to URL', e.reason.args[1])
+                reraise_exception(err, sys.exc_info())
             try:
                 reason = e.reason.args[1]
             except (AttributeError, IndexError):
                 reason = e.reason
-            reraise_exception(HTTPClientError('No response from URL "%s": %s'
-                                              % (url, reason)), sys.exc_info())
+            err = self.handle_url_exception(url, 'No response from URL', reason)
+            reraise_exception(err, sys.exc_info())
         except ValueError as e:
-            reraise_exception(HTTPClientError('URL not correct "%s": %s'
-                                              % (url, e.args[0])), sys.exc_info())
+            err = self.handle_url_exception(url, 'URL not correct', e.args[0])
+            reraise_exception(err, sys.exc_info())
         except Exception as e:
-            reraise_exception(HTTPClientError('Internal HTTP error "%s": %r'
-                                              % (url, e)), sys.exc_info())
+            err = self.handle_url_exception(url, 'Internal HTTP error', repr(e))
+            reraise_exception(err, sys.exc_info())
         else:
             code = getattr(result, 'code', 200)
             if code == 204:
@@ -224,8 +234,24 @@ class HTTPClient(object):
                 raise HTTPClientError('response is not an image: (%s)' % (resp.read()))
         return ImageSource(resp)
 
+    def handle_url_exception(self, url, message, reason, response_code=None):
+        full_msg = '%s "%s": %s' % (message, url, reason)
+        if self.hide_error_details:
+            return HTTPClientError(
+                '{} (see logs for URL and reason).'.format(message),
+                response_code=response_code,
+                full_msg=full_msg,
+            )
+        else:
+            return HTTPClientError(
+                full_msg,
+                response_code=response_code,
+            )
+
 def auth_data_from_url(url):
     """
+    >>> auth_data_from_url('invalid_url')
+    ('invalid_url', (None, None))
     >>> auth_data_from_url('http://localhost/bar')
     ('http://localhost/bar', (None, None))
     >>> auth_data_from_url('http://bar@localhost/bar')
@@ -238,16 +264,35 @@ def auth_data_from_url(url):
     ('http://localhost/bar', ('bar foo; foo@bar', 'b:az@'))
     >>> auth_data_from_url('https://bar:foo#;%$@localhost/bar')
     ('https://localhost/bar', ('bar', 'foo#;%$'))
+    >>> auth_data_from_url('http://localhost/bar@2x')
+    ('http://localhost/bar@2x', (None, None))
+    >>> auth_data_from_url('http://bar@localhost/bar@2x')
+    ('http://localhost/bar@2x', ('bar', None))
+    >>> auth_data_from_url('http://bar:baz@localhost/bar@2x')
+    ('http://localhost/bar@2x', ('bar', 'baz'))
+    >>> auth_data_from_url('https://bar@localhost/bar/0/0/0@2x.png')
+    ('https://localhost/bar/0/0/0@2x.png', ('bar', None))
+    >>> auth_data_from_url('http://bar:baz@localhost/bar@2x.png')
+    ('http://localhost/bar@2x.png', ('bar', 'baz'))
     """
+    if not url or '://' not in url:
+        # be silent for invalid URLs
+        return url, (None, None)
+
+    schema, url = url.split('://', 1)
+    if '/' in url:
+        host, request = url.split('/', 1)
+    else:
+        host, request = url, ''
+
     username = password = None
-    if '@' in url:
-        head, url = url.rsplit('@', 1)
-        schema, auth_data = head.split('//', 1)
-        url = schema + '//' + url
+    if '@' in host:
+        auth_data, host = host.rsplit('@', 1)
         if ':' in auth_data:
             username, password = auth_data.split(':', 1)
         else:
             username = auth_data
+    url = schema + "://" + host + "/" + request
     return url, (username, password)
 
 
