@@ -20,56 +20,60 @@ from mapproxy.test.mocker import ANY
 from mapproxy.response import Response
 from mapproxy.compat import string_type
 
+
 class TestResponse(Mocker):
+
     def test_str_response(self):
-        resp = Response('string content')
+        resp = Response("string content")
         assert isinstance(resp.response, string_type)
         start_response = self.mock()
-        self.expect(start_response('200 OK', ANY))
+        self.expect(start_response("200 OK", ANY))
         self.replay()
-        result = resp({'REQUEST_METHOD': 'GET'}, start_response)
-        assert next(result) == b'string content'
+        result = resp({"REQUEST_METHOD": "GET"}, start_response)
+        assert next(result) == b"string content"
 
     def test_itr_response(self):
-        resp = Response(iter(['string content', 'as iterable']))
-        assert hasattr(resp.response, 'next') or hasattr(resp.response, '__next__')
+        resp = Response(iter(["string content", "as iterable"]))
+        assert hasattr(resp.response, "next") or hasattr(resp.response, "__next__")
         start_response = self.mock()
-        self.expect(start_response('200 OK', ANY))
+        self.expect(start_response("200 OK", ANY))
         self.replay()
-        result = resp({'REQUEST_METHOD': 'GET'}, start_response)
-        assert next(result) == 'string content'
-        assert next(result) == 'as iterable'
+        result = resp({"REQUEST_METHOD": "GET"}, start_response)
+        assert next(result) == "string content"
+        assert next(result) == "as iterable"
 
     def test_file_response(self):
-        data = BytesIO(b'foobar')
+        data = BytesIO(b"foobar")
         resp = Response(data)
         assert resp.response == data
         start_response = self.mock()
-        self.expect(start_response('200 OK', ANY))
+        self.expect(start_response("200 OK", ANY))
         self.replay()
-        result = resp({'REQUEST_METHOD': 'GET'}, start_response)
-        assert next(result) == b'foobar'
+        result = resp({"REQUEST_METHOD": "GET"}, start_response)
+        assert next(result) == b"foobar"
 
     def test_file_response_w_file_wrapper(self):
-        data = BytesIO(b'foobar')
+        data = BytesIO(b"foobar")
         resp = Response(data)
         assert resp.response == data
         start_response = self.mock()
-        self.expect(start_response('200 OK', ANY))
+        self.expect(start_response("200 OK", ANY))
 
         file_wrapper = self.mock()
-        self.expect(file_wrapper(data, resp.block_size)).result('DUMMY')
+        self.expect(file_wrapper(data, resp.block_size)).result("DUMMY")
         self.replay()
 
-        result = resp({'REQUEST_METHOD': 'GET',
-                       'wsgi.file_wrapper': file_wrapper}, start_response)
-        assert result == 'DUMMY'
+        result = resp(
+            {"REQUEST_METHOD": "GET", "wsgi.file_wrapper": file_wrapper}, start_response
+        )
+        assert result == "DUMMY"
+
     def test_file_response_content_length(self):
-        data = BytesIO(b'*' * 342)
+        data = BytesIO(b"*" * 342)
         resp = Response(data)
         assert resp.response == data
         start_response = self.mock()
-        self.expect(start_response('200 OK', ANY))
+        self.expect(start_response("200 OK", ANY))
         self.replay()
-        resp({'REQUEST_METHOD': 'GET'}, start_response)
+        resp({"REQUEST_METHOD": "GET"}, start_response)
         assert resp.content_length == 342
