@@ -16,13 +16,14 @@
 """
 WMS clients for maps and information.
 """
-from __future__ import with_statement
+import sys
+
 from mapproxy.compat import text_type
 from mapproxy.request.base import split_mime_type
 from mapproxy.layer import InfoQuery
 from mapproxy.source import SourceError
 from mapproxy.client.http import HTTPClient
-from mapproxy.srs import make_lin_transf, SRS
+from mapproxy.srs import make_lin_transf, SRS, SupportedSRS
 from mapproxy.image import ImageSource
 from mapproxy.image.opts import ImageOptions
 from mapproxy.featureinfo import create_featureinfo_doc
@@ -40,6 +41,7 @@ class WMSClient(object):
         self.fwd_req_params = fwd_req_params or set()
 
     def retrieve(self, query, format):
+        log.debug(query)
         if self.http_method == 'POST':
             request_method = 'POST'
         elif self.http_method == 'GET':
@@ -73,10 +75,19 @@ class WMSClient(object):
                 log_size = 8000 # larger xml exception
             else:
                 log_size = 100 # image?
-            data = resp.read(log_size)
-            if len(data) == log_size:
-                data += '... truncated'
-            log.warn("no image returned from source WMS: %s, response was: %s" % (url, data))
+            data = resp.read(log_size+1)
+
+            truncated = ''
+            if len(data) == log_size+1:
+                data = data[:-1]
+                truncated = ' [output truncated]'
+
+            if sys.version_info >= (3, 5, 0):
+                data = data.decode('utf-8', 'backslashreplace')
+            else:
+                data = data.decode('ascii', 'ignore')
+
+            log.warning("no image returned from source WMS: {}, response was: '{}'{}".format(url, data, truncated))
             raise SourceError('no image returned from source WMS: %s' % (url, ))
 
     def _query_url(self, query, format):
@@ -92,7 +103,6 @@ class WMSClient(object):
         req.params.size = query.size
         req.params.srs = query.srs.srs_code
         req.params.format = format
-        # also forward dimension request params if available in the query
         req.params.update(query.dimensions_for_params(self.fwd_req_params))
         return req
 
@@ -116,15 +126,21 @@ class WMSInfoClient(object):
         self.request_template = request_template
         self.http_client = http_client or HTTPClient()
         if not supported_srs and self.request_template.params.srs is not None:
-            supported_srs = [SRS(self.request_template.params.srs)]
-        self.supported_srs = supported_srs or []
+            supported_srs = SupportedSRS([SRS(self.request_template.params.srs)])
+        self.supported_srs = supported_srs
 
     def get_info(self, query):
         if self.supported_srs and query.srs not in self.supported_srs:
             query = self._get_transformed_query(query)
         resp = self._retrieve(query)
-        info_format = resp.headers.get('Content-type', None)
+
+        # use from template if available
+        info_format = self.request_template.params.get('info_format')
         if not info_format:
+            # otherwise from response
+            info_format = resp.headers.get('Content-type', None)
+        if not info_format:
+            # otherwise from query
             info_format = query.info_format
         return create_featureinfo_doc(resp.read(), info_format)
 
@@ -134,18 +150,21 @@ class WMSInfoClient(object):
         """
         req_srs = query.srs
         req_bbox = query.bbox
-        info_srs = self._best_supported_srs(req_srs)
-        info_bbox = req_srs.transform_bbox_to(info_srs, req_bbox)
+        req_coord = make_lin_transf((0, 0, query.size[0], query.size[1]), req_bbox)(query.pos)
 
-        req_coord = make_lin_transf((0, query.size[1], query.size[0], 0), req_bbox)(query.pos)
+        info_srs = self.supported_srs.best_srs(req_srs)
+        info_bbox = req_srs.transform_bbox_to(info_srs, req_bbox)
+        # calculate new info_size to keep square pixels after transform_bbox_to
+        info_aratio = (info_bbox[3] - info_bbox[1])/(info_bbox[2] - info_bbox[0])
+        info_size = query.size[0], int(info_aratio*query.size[0])
 
         info_coord = req_srs.transform_to(info_srs, req_coord)
-        info_pos = make_lin_transf((info_bbox), (0, query.size[1], query.size[0], 0))(info_coord)
+        info_pos = make_lin_transf((info_bbox), (0, 0, info_size[0], info_size[1]))(info_coord)
         info_pos = int(round(info_pos[0])), int(round(info_pos[1]))
 
         info_query = InfoQuery(
             bbox=info_bbox,
-            size=query.size,
+            size=info_size,
             srs=info_srs,
             pos=info_pos,
             info_format=query.info_format,
@@ -153,10 +172,6 @@ class WMSInfoClient(object):
         )
         return info_query
 
-    def _best_supported_srs(self, srs):
-        # always choose the first, distortion should not matter
-        return self.supported_srs[0]
-
     def _retrieve(self, query):
         url = self._query_url(query)
         return self.http_client.open(url)
